#!/bin/sh
# the next line restarts using wish \
exec wishstep8.0 "$0" "$@"

#####################################
# icon binding for apps
#

proc ttb_appIconInfo name {

	global ttblibPath

	if [file exists $ttblibPath/ttb.apps] {
		set ttbapps [open $ttblibPath/ttb.apps r]
		set line [split [gets $ttbapps] \;]

		while {![eof $ttbapps]} {

			if {$name==[lindex $line 0]} {
				return [lindex $line 1]
			}

		        set line [split [gets $ttbapps] \;]
		}
		close $ttbapps
		return Execute
	}
}

##########################################
#
# Item binding
#

proc ttb_makeDefaultItem {} {

	global ttblibPath itemWidth itemHeight stageWidth stageHeight panelColor stageColor
	# set default shortcuts

	frame .container.shortcuts.home -width $itemWidth -height $itemHeight -bg $panelColor
	pack .container.shortcuts.home -in .container.shortcuts -side left

	image create photo shortcuthome -file "$ttblibPath/icons/shortCuts/Home.gif"

	label .container.shortcuts.home.icon -image shortcuthome -bd 0 -bg $panelColor
	label .container.shortcuts.home.fname -text [exec whoami] \
		-width 16 -height 2 -anchor n -wraplength 120 -bg $panelColor

	pack .container.shortcuts.home.icon .container.shortcuts.home.fname -in .container.shortcuts.home

	bind .container.shortcuts.home.icon <Double-ButtonPress-1> "ttb_showFiles 1 ~/"

	bind .container.shortcuts.home.fname <Double-ButtonPress-1> "ttb_showFiles 1 ~/"

}

###########################################
# file browse
#

proc ttb_mkAlias {fname} {
	if {[file isfile $fname]==1} {
		exec ln -s $fname "$fname.aliasFile"
	} else {
		exec ln -s $fname "$fname.aliasFolder"
	}
}

proc ttb_mkCopy {fname} {
	if {[file isfile $fname]==1} {
		file copy -force -- $fname "$fname.copyFile"
	} else {
		file copy -force -- $fname "$fname.copyFolder"
	}
}

proc sub_renameFile {fname} {
	set newName [.inputNewName.enter get]

	file rename -force -- $fname $newName
	destroy .inputNewName
}

proc ttb_renameFile {fname x y} {
	global ttblibPath stageColor newName panelColor

	set newName $fname

	toplevel .inputNewName -bd 10 -relief flat -bg $panelColor
	wm geometry .inputNewName +$x+$y
	grab .inputNewName

	wm title .inputNewName "Enter new name"

	label .inputNewName.label -text "Enter new file name" -bd 0 -bg $panelColor

	entry .inputNewName.enter -width 64 -relief sunken -bd 2 -textvariable newName
	pack .inputNewName.label .inputNewName.enter -fill both

	bind .inputNewName.enter <Any-Return>  "sub_renameFile $fname"
}

proc sub_touchFile {mode} {
	set touchName [.inputTouchName.enter get]

	switch $mode {
		"file" {
			exec touch $touchName
		}
		"folder" {
			catch "file mkdir $touchName"
		}
	}

	destroy .inputTouchName
}

proc ttb_touchFile {mode x y} {
	global ttblibPath stageColor newName panelColor

	toplevel .inputTouchName -bd 10 -relief flat -bg $panelColor
	wm geometry .inputTouchName +$x+$y
	grab .inputTouchName

	wm title .inputTouchName "Enter file name"

	label .inputTouchName.label -text "Enter new file name" -bd 0 -bg $panelColor

	entry .inputTouchName.enter -width 32 -relief sunken -bd 2
	pack .inputTouchName.label .inputTouchName.enter -fill both

	bind .inputTouchName.enter <Any-Return>  "sub_touchFile $mode"
}

proc ttb_deleteFile {filename x y} {

	if [file isdirectory $filename] {
		set ret [ttb_dialog .deleteok "File Delete" \
			"Folder:$filename delete?" ttbLogo $x $y 1 Ok Cancel ]
	} else {
		set ret [ttb_dialog .deleteok "File Delete" \
			"File:$filename delete?" ttbLogo $x $y 1 Ok Cancel]
	}

	if {$ret == 0} {file delete -force -- $filename} else {return}
}

proc sub_showFiles_truncateFilename {} {

	if {[string length [lindex [split $fileList \;] [expr $j+4*$i]]] > 30} {
		set pathList [split [string trim $pwd "/"] "/"]
		set len [llength $pathList]
		set prev ""
		for {set i $len} {1} {incr i -1} {
			set elem [lindex $pathList [expr $i -1]]
			set cur "$elem/$prev"
			if {[string length $cur] > 25} {
				if {$i == $len} { set prev $elem}
				set winTitle "/.../$prev"
				break;
			}
			set prev $cur
		}
	}
}

proc sub_showFiles_raiseAndChange pwd {cd $pwd}

proc sub_showFiles_DnD {channel} {

	if {$channel==2} {
		set fileList [.hidedBuffer2 get 0]
	} else {
		set fileList [.hidedBuffer get $channel]
	}

	set flLen [llength $fileList]

	if {$flLen==0} {
		return
	} 

	set fileList [join $fileList \n]

	if {$flLen==1} {
		dnd_setdata File $fileList
	} else {
		dnd_setdata Files $fileList
	}

	dnd_handledrag
}

proc sub_showFiles_selectFile {canvas id fname mode channel} {
	global slColor stageColor oldFill selectedFilesID fileList

	.hidedBuffer delete $channel

	set fList {}

	foreach fName [join [split $fileList \;]] {
		catch [set fList [concat $fList [pwd]/$fName]]
	}

	set selectedFilesList {}

	switch $mode {
		"sel" {
			$canvas itemconfigure $id -fill $slColor
			set oldFill $slColor

			if {[info exists selectedFilesID($id)] == 0} {
				set selectedFilesID($id) $fname
			}
		}

		"unsel" {
			$canvas itemconfigure $id -fill $stageColor
			set oldFill $stageColor

			if {[info exists selectedFilesID($id)] == 1} {
				unset selectedFilesID($id)
			}
		}

		"selall" {
			set IDs [$canvas find all]

			set j 0
			for {set i 0} {$i<[llength $IDs]} {incr i} {

				set ID [$canvas gettags [lindex $IDs $i]]

				if {[string first txt $ID]==0} {incr i;continue}
				if {[string first img $ID]==0} {incr i;continue}

				$canvas itemconfigure $ID -fill $slColor

				set oldFill $slColor

				if {[info exists selectedFilesID($ID)] == 0} {
					set selectedFilesID($ID) [lindex $fList $j]
				}

				incr j
			}

			set selectedFileList $fList
			.hidedBuffer insert $channel $fList
			.hidedBuffer2 insert 0 $fList
			return
		}

 		"clear" {
			foreach id [array names selectedFilesID] {
				if {[info exists selectedFilesID($id)] == 1} {
					$canvas itemconfigure $id -fill $stageColor
					set oldFill $stageColor
					unset selectedFilesID($id)
				}
			}

			set  selectedFilesList {}
			.hidedBuffer delete $channel
			return
		}
	}

	foreach file [array names selectedFilesID] {
		catch [set selectedFilesList [concat $selectedFilesList $selectedFilesID($file)]]
	}

	.hidedBuffer insert $channel "$selectedFilesList"
	set  selectedFilesList {}
}

proc sub_showFiles_scrollEnter {canvas id mode} {
	global oldFill hlColor mlColor slColor

	set oldFill [lindex [$canvas itemconfig $id -fill] 4]

	if {$mode == "m"} {
		$canvas itemconfigure $id -fill $mlColor
	} elseif {$mode == "h"} {
		$canvas itemconfigure $id -fill $hlColor
	} else {
		$canvas itemconfigure $id -fill $slColor
	}
}

proc sub_showFiles_scrollLeave {canvas id} {
	global oldFill

	$canvas itemconfigure $id -fill $oldFill
}

proc sub_showFiles_showItems {tl i j fileName nameBase mode} {

	global itemWidth itemHeight stageWidth stageHeight stageColor

	set geomX [expr $j*$itemWidth]
	set geomY [expr $i*$itemHeight]
	set hitemWidth [expr $itemWidth/2]
	set hitemHeight [expr $itemHeight/2]

	if {$mode==1} {
		set geomX [expr $geomX+8*$j] 
		$tl create image \
			[expr $geomX-4] [expr $geomY+36] -image NeXTarrowR
	}

	$tl create rectangle \
		$geomX [expr $geomY+8] \
		[expr $geomX+$itemWidth] [expr $geomY+$itemHeight] \
		-rounded 10 -fill $stageColor -outline $stageColor -tags $i\_$j

	$tl create image \
		[expr $geomX+$hitemWidth] \
		[expr $geomY+$hitemHeight-8] \
		-image $nameBase$i\_$j -tags img$i\_$j

	$tl create text \
		[expr $geomX+$hitemWidth] \
		[expr $geomY+$hitemHeight+24] \
		-width $itemWidth -justify center \
		-text $fileName -tags txt$i\_$j

	$tl bind $i\_$j <Any-Enter> "sub_showFiles_scrollEnter $tl $i\_$j m"
	$tl bind $i\_$j <Any-Leave> "sub_showFiles_scrollLeave $tl $i\_$j"

	$tl bind img$i\_$j <Any-Enter> "sub_showFiles_scrollEnter $tl $i\_$j h"
	$tl bind img$i\_$j <Any-Leave> "sub_showFiles_scrollLeave $tl $i\_$j"


	$tl bind txt$i\_$j <Any-Enter> "sub_showFiles_scrollEnter $tl $i\_$j h"
	$tl bind txt$i\_$j <Any-Leave> "sub_showFiles_scrollLeave $tl $i\_$j"

	return $i\_$j
}

proc sub_ttb_showFiles_reshowCheck {oldGeom} {

	global stageWidth stageHeight itemXMax itemWidth

	if {$oldGeom != [winfo width .]} {
		set stageWidth [expr [winfo width .]-32]
		set stageHeight [expr [winfo height .]-200]
		set itemXMax [expr $stageWidth/$itemWidth]

		ttb_showFiles 1 [pwd]
	}

	set oldGeom [winfo width .]

	global oldList fileList newdir

	set oldList $fileList

	sub_showFiles_filter

	set numberOfFiles [llength [split $fileList \;]]

	if { [string compare $oldList $fileList] != 0 } {
		ttb_showFiles 1 [pwd]
	}

	after [expr 1000+20*$numberOfFiles] "sub_ttb_showFiles_reshowCheck $oldGeom"
}

proc sub_ttb_showFiles_reshowFolderWindow {} {

	global ttblibPath itemXMax itemWidth itemHeight stageColor env_pathList stageWidth stageHeight fileList

	.container.folderStage delete all

	set pwd [pwd]
	set pathList [split [string trim $pwd "/"] "/"]
	set modlist {{} img txt}

	bind .container.folderStage <ButtonPress-3> "\
		.container.folderStage scan mark %X %Y;ttb_showPopup top %X %Y {}"

	.container.folderStage configure \
		-scrollregion "0 0 [expr [llength $pathList]*$itemWidth+$itemWidth*1.5] 0"

	if {[winfo exists .container.folderStage]} {

		bind .container.folderStage <2> ".container.folderStage scan mark %x %y"
		bind .container.folderStage <B2-Motion> ".container.folderStage scan dragto %x %y"

		image create photo ficn0_0 -file "$ttblibPath/icons/folders/computer.black.gif"

		set ID [sub_showFiles_showItems .container.folderStage 0 0 [exec hostname] ficn 0]

		foreach mod $modlist {
			.container.folderStage bind $mod$ID <Double-ButtonPress-1> "ttb_showFiles 1 /"
			.container.folderStage bind $mod$ID <Drag> "sub_showFiles_DnD 1"
		}

		for {set i 1} {$i<[expr [llength $pathList] + 1]} {incr i} {

			if {[lindex $pathList [expr $i-1]] == [exec whoami]} {
				image create photo ficn0\_$i -file "$ttblibPath/icons/folders/Home.gif"
			} else {
				image create photo ficn0\_$i -file "$ttblibPath/icons/folders/Folder.gif"
			}

			set absPath {}

			for {set k 0} {$k<$i} {incr k} {
				catch [set absPath [concat $absPath [lindex $pathList $k]]]
			}

			set absPath /[join $absPath /]

			set ID [sub_showFiles_showItems .container.folderStage 0 $i [lindex $pathList [expr $i-1]] ficn 1]

			foreach mod $modlist {
				.container.folderStage bind $mod$ID <Double-ButtonPress-1> "ttb_showFiles 1 $absPath"
				.container.folderStage bind $mod$ID <Drag> "sub_showFiles_DnD 1"
			}
		}
	}
	.container.folderStage xview moveto 1.0
}

proc ttb_do_stageDrop {pwd fileList} {
	if {[join [.hidedBuffer2 get 0] \n]==$fileList} {
		for {set i 0} {$i<[llength $fileList]} {incr i} {
			eval [.clipBoard.modeList get $i] [lindex $fileList $i] $pwd
		}
	} else {
		set fileList [join [split $fileList \n]]
		foreach i $fileList {
			eval file copy -force -- $i $pwd
		}
	}

	.hidedBuffer delete 1 end
	.hidedBuffer2 delete 0 end

	if {[winfo exists .clipBoard]} {
		.clipBoard.fileList delete 0 end
		.clipBoard.modeList delete 0 end
	}
}

proc sub_ttb_showFiles_reshowFileWindow {} {

	global ttblibPath itemXMax itemWidth itemHeight stageColor env_pathList stageWidth stageHeight fileList
	.container.stage delete all

	set hit 0
	set pwd [pwd]

	foreach i $env_pathList {if {$pwd==$i} {set hit 1}}

	set numberOfFiles [llength [split $fileList \;]]
	if {[expr $numberOfFiles%4] == 0} {
		set itemYMax [expr $numberOfFiles/$itemXMax+1]
	} else {
		set itemYMax [expr $numberOfFiles/$itemXMax+2]
	}


#	bind .container.stage <2> ".container.stage scan mark %x %y"
#	bind .container.stage <B2-Motion> ".container.stage scan dragto %x %y"
	bind .container.stage <Drop> "ttb_do_stageDrop $pwd %D"
	bind .container.stage <Shift-ButtonPress-3> "\
		.container.stage scan mark %X %Y;ttb_showPopup stage %X %Y {}"

	.container.stage configure \
		-scrollregion "0 0 [expr $itemXMax*$itemWidth] [expr $itemYMax*$itemHeight]"

	set modlist {{} img txt}

	for {set i 0} {$i<$itemYMax} {incr i} {
		for {set j 0} {$j<$itemXMax} {incr j} {

			set fileName [lindex [split $fileList \;] [expr $j+$itemXMax*$i]]

			if {[file exists $fileName]} {
				if {[file isdirectory $fileName]} {

					if {$fileName == [exec whoami]} {
						image create photo icn$i\_$j -file "$ttblibPath/icons/folders/Home.gif"
					} elseif {[file extension $fileName]==".aliasFolder"} {
						image create photo icn$i\_$j -file "$ttblibPath/icons/folders/FolderLink.gif"
					} elseif {[file extension $fileName]==".copyFolder"} {
						image create photo icn$i\_$j -file "$ttblibPath/icons/folders/FolderLink.gif"
					} else {
						image create photo icn$i\_$j -file "$ttblibPath/icons/folders/Folder.gif"
					}

					set ID [sub_showFiles_showItems .container.stage $i $j $fileName icn 0]

					foreach mod $modlist {
						.container.stage bind $mod$ID <Double-ButtonPress-1> \
"ttb_execIF2 $fileName 0"
						.container.stage bind $mod$ID <Drag> "sub_showFiles_DnD 0"
					}

					.container.stage bind img$ID <ButtonPress-1> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName sel 0\
					"

					.container.stage bind img$ID <ButtonPress-2> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName sel 0\
					"

					.container.stage bind img$ID <Shift-ButtonPress-1> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName sel 0\
                                        "

					.container.stage bind img$ID <Control-ButtonPress-1> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName unsel 0\
					"

					.container.stage bind img$ID <ButtonPress-3> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						.container.stage scan mark %X %Y;\
						ttb_showPopup folder %X %Y\
						$pwd/$fileName\
					"
				} else {
					if {$hit == 1} {
						image create photo icn$i\_$j -file \
							$ttblibPath/icons/app/[ttb_appIconInfo $fileName].gif
					} else {
						image create photo icn$i\_$j -file \
							$ttblibPath/icons/files/[ttb_execIF2 $fileName 2].gif
					}
 
					set ID [sub_showFiles_showItems .container.stage $i $j $fileName icn 0]

					foreach mod $modlist {
						.container.stage bind $mod$ID <Double-ButtonPress-1> "ttb_execIF2 $fileName 0"
						.container.stage bind $mod$ID <Double-ButtonPress-2> "ttb_execIF2 $fileName 1"
						.container.stage bind $mod$ID <Drag> "sub_showFiles_DnD 0"
					}

					.container.stage bind img$ID <ButtonPress-1> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName sel 0\
					"

					.container.stage bind img$ID <ButtonPress-2> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName sel 0\
					"

					.container.stage bind img$ID <Shift-ButtonPress-1> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName sel 0\
                                        "

					.container.stage bind img$ID <Control-ButtonPress-1> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j $pwd/$fileName unsel 0\
					"

					.container.stage bind img$ID <ButtonPress-3> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						.container.stage scan mark %X %Y;\
						ttb_showPopup file %X %Y\
						$pwd/$fileName\
					"
				}
			} else {
				if {[catch "file readlink $fileName"]==0} {
					image create photo icn$i\_$j -file "$ttblibPath/icons/system/BulletHole.gif"
					set ID [sub_showFiles_showItems .container.stage $i $j BrokenLink icn 0]

					foreach mod $modlist {
						.container.stage bind $mod$ID <Double-ButtonPress-1> \
							"ttb_deleteFile $fileName %X %Y"
						.container.stage bind $mod$ID <Double-ButtonPress-2> \
							"ttb_deleteFile $fileName %X %Y"
						.container.stage bind $mod$ID <Drag> "sub_showFiles_DnD 0"
					}

					.container.stage bind img$ID <ButtonPress-3> "\
						sub_showFiles_selectFile .container.stage \
						$i\_$j {} clear 0;\
						ttb_showPopup broken %X %Y\
						$pwd/$fileName\
					"
				}
			}
		}
	}
}

proc sub_showFiles_filter {} {

	global filt sort fileList selectedFilesList selectedFilesID

	set fileList {}

	switch $filt {
		"0" {
			if [catch {set files [lsort [glob .*]]}] {set files {} }
			catch {set files [concat $files [lsort [glob *]]]}
		}

		"1" {
			set files {}
			catch {set files [concat $files [lsort [glob *]]]}
		}

		"2" {
			set files {}
			catch {set files [concat $files [lsort [glob *]]]}

			foreach file $files {
				if {[string match "*~" $file]==0} {
					catch [set fileList [concat $fileList $file]]
				}
			}

			unset files
			set fileList [join $fileList \;]
			return
		}

		"3" {
			set files {}
			catch {set files [concat $files [lsort [glob *~]]]}
		}
	}

	foreach file $files {
		if {$file != "."} {
			if {$file != ".."} {
				catch [set fileList [concat $fileList $file]]
			}
		}
	}

	unset files

	set tempList {}

	switch $sort {
		"0" {
			set tempList $fileList
		}

		"1" {
			foreach i $fileList {
				if {[file isdirectory $i]} {
					catch [set tempList [concat $tempList $i]]
				}
			}

			foreach i $fileList {
				if {[file isfile $i]} {
					catch [set tempList [concat $tempList $i]]
				}
			}
		}

		"2" {
			foreach i $fileList {
				if {[file isfile $i]} {
					catch [set tempList [concat $tempList $i]]
				}
			}

			foreach i $fileList {
				if {[file isdirectory $i]} {
					catch [set tempList [concat $tempList $i]]
				}
			}
		}
	}

	set fileList [join $tempList \;]
}

proc ttb_showFiles {mode newdir} {

	global fileList selectedFilesList selectedFilesID

	set oldGeom [winfo width .]

	foreach file [array names selectedFilesID] {
		unset selectedFilesID($file)
	}    

	cd $newdir

	sub_showFiles_filter

	sub_ttb_showFiles_reshowFolderWindow
	sub_ttb_showFiles_reshowFileWindow

	if {$mode==0} {
		sub_ttb_showFiles_reshowCheck $oldGeom
	}
}

###################################
#
# exit TTB
#

proc ttb_exit {} {

	set ret [ttb_dialog .deleteok "Exit TTB" \
		"Exit File Viewer?" ttbLogo -32768 0 0 Ok Cancel ]

	if {$ret == 0} {exit} else {return}
}

###################################
# configurate popup menu
#
proc ttb_showPopup {mode x y fname} {
	global ttbsendList ttbbookmarkList filt

	if {[winfo exists .ttb_popup]} {
		destroy .ttb_popup
		destroy .ttb_popup.send .ttb_popup.bookmark .ttb_popup.filter

	}

	menu .ttb_popup -tearoff 0
	menu .ttb_popup.send -tearoff 0
	menu .ttb_popup.bookmark -tearoff 0
	menu .ttb_popup.filter -tearoff 0
	menu .ttb_popup.sorter -tearoff 0

	foreach send $ttbsendList {
		.ttb_popup.send add command -label $send -command "exec $send $fname &"
	}

	foreach bookmark $ttbbookmarkList {
		.ttb_popup.bookmark add command -label $bookmark -command "ttb_showFiles 1 $bookmark"
	}

	set filters {{Show all files} {Hide dot files} {Show only normal} {Show only backup}}

	for {set filter 0} {$filter<4} {incr filter} { 
		.ttb_popup.filter add command -label [lindex $filters $filter] -command "set filt $filter"
	}

	set sorters {{Alphabetical} {Folder->File} {File->Folder}}

	for {set sorter 0} {$sorter<3} {incr sorter} { 
		.ttb_popup.sorter add command -label [lindex $sorters $sorter] -command "set sort $sorter"
	}

	switch $mode {
		"top" {
			.ttb_popup add command -label "About Jun's STEP" -command "ttb_about"
			.ttb_popup add command -label "Processes" -command "ttb_proclist"
			.ttb_popup add command -label "Item window" -command ttb_showItems
			.ttb_popup add command -label "exit" -command ttb_exit
		}

		"file" {
			.ttb_popup add command -label "rename" \
				-command "ttb_renameFile $fname $x $y"
			.ttb_popup add command -label "view or exec" \
				-command "ttb_execIF2 $fname 0"
			.ttb_popup add command -label "edit" \
				-command "ttb_execIF2 $fname 1"
			.ttb_popup add cascade -label "send to" \
				-menu .ttb_popup.send
			.ttb_popup add command -label "Make alias" \
				-command "ttb_mkAlias $fname"
			.ttb_popup add command -label "Make copy" \
				-command "ttb_mkCopy $fname"
			.ttb_popup add command -label "delete" \
				-command "ttb_deleteFile $fname $x $y"
			.ttb_popup add command -label "UNIX permission" \
				-command "ttb_unixPermission $fname"
		}

		"folder" {
			.ttb_popup add command -label "rename" \
				-command "ttb_renameFile $fname $x $y"
			.ttb_popup add command -label "open" \
				-command "ttb_showFiles 1 $fname"
			.ttb_popup add command -label "Make alias" \
				-command "ttb_mkAlias $fname"
			.ttb_popup add command -label "Make copy" \
				-command "ttb_mkCopy $fname"
			.ttb_popup add command -label "delete" \
				-command "ttb_deleteFile $fname $x $y"
			.ttb_popup add command -label "UNIX permission" \
				-command "ttb_unixPermission $fname"
		}

		"stage" {
			.ttb_popup add command -label "Parent" -command "ttb_showFiles 1 .."

			.ttb_popup add cascade -label "bookmarks" \
				-menu .ttb_popup.bookmark

			.ttb_popup add command -label "Select all" -command {
					sub_showFiles_selectFile .container.stage {} {} selall 0
			}

			.ttb_popup add command -label "Unselect all" -command {
					sub_showFiles_selectFile .container.stage {} {} clear 0
			}

			.ttb_popup add command -label "Make new file" -command "ttb_touchFile file $x $y"
			.ttb_popup add command -label "Make new folder" -command "ttb_touchFile folder $x $y"
			.ttb_popup add command -label "Item window" -command ttb_showItems
			.ttb_popup add cascade -label "Filter" -menu .ttb_popup.filter
			.ttb_popup add cascade -label "File Sort" -menu .ttb_popup.sorter
			.ttb_popup add command -label "exit" -command ttb_exit
		}

		"broken" {
			.ttb_popup add command -label "delete" -command "ttb_deleteFile $fname $x $y"
		}
	}
	tk_popup .ttb_popup $x $y
}

proc ttb_about {} {

	ttb_dialog .about "about File Viewer" \
		"Jun's STEP File Viewer\n\nver 1.3\n\nJun Wakaya 1998."\
		jfLogo -32768 0 0 Ok

}

#################################
# reading startup file
#
proc ttb_readStartup {} {
	global ttblibPath ttblogo ttbx ttby ttbTerm stageColor panelColor hlColor mlColor
	global ttbsendList ttbbookmarkList

	if [file exists ~/GNUstep/Library/JunsSTEP/ttbinitrc] {
		set ttbrc [open ~/GNUstep/Library/JunsSTEP/ttbinitrc r]
		set line [split [gets $ttbrc]]

		while {![eof $ttbrc]} {
			switch [lindex $line 0] {
				ttblibPath {
					set ttblibPath [lindex $line 1]
				}
				ttblogo {
					set ttblogo [lindex $line 1]
				}
				ttbx {
					set ttbx [lindex $line 1]
				}
				ttby {
					set ttby [lindex $line 1]
				}
				ttbTerm {
					set ttbTerm [lindex $line 1]
				}
				ttbbg {
					set stageColor [lindex $line 1]
				}
				ttbfg {
					set panelColor [lindex $line 1]
				}
				ttbMidlight {
					set mlColor [lindex $line 1]
				}
				ttbHighlight {
					set hlColor [lindex $line 1]
				}
			}
	        set line [split [gets $ttbrc]]
		}
		close $ttbrc

		if [file exists $ttblibPath/ttb.send] {
			set ttbsend [open $ttblibPath/ttb.send r]

			while {![eof $ttbsend]} {
				catch {set ttbsendList [concat $ttbsendList $line]}
				set line [split [gets $ttbsend]]
			}
		}

		if [file exists $ttblibPath/ttb.bookmark] {
			set ttbbookmark [open $ttblibPath/ttb.bookmark r]

			while {![eof $ttbbookmark]} {
				catch {set ttbbookmarkList [concat $ttbbookmarkList $line]}
				set line [split [gets $ttbbookmark]]
			}
		}
	} else {
		source ../Library/functions/ttb_installer.tcl
		ttb_install
		ttb_readStartup
	}
}

################################
#
#

proc ttb_commonBinding {} {
	bind . <q> ttb_exit
	bind . <p> "ttb_showFiles 1 .."
}

##################################
# main routine
#

set env_pathList {}

foreach i [join [split [split $env(PATH) :] .]] {
	if {$i!={}} {
		if {$i!=" "} {
			set temp [split [string trim $i /] /]

			for {set j 0} {$j<[llength $temp]} {incr j} {

				if {[lindex $temp $j]=="X11"} {
					set temp [lreplace $temp $j $j "X11R6"]
				}
			}
			catch [set env_pathList [concat $env_pathList /[join $temp /]]]
		}
	}
}

set ttblibPath "~/GNUstep/Library/JunsSTEP/"
set ttblogo "JFLOGO"
set ttbx 640
set ttby 480
set ttbTerm xterm
set itemXMax 0
set stageColor #bfbfbf
set panelColor #bfbfbf
set hlColor #ffffff
set mlColor #dddddd
set slColor #ddffaa
set ttbsendList {}
set ttbbookmarkList {}
set filt 1
set sort 1
set selectedFilesList {}
array names selectedFilesID
set lock 0

ttb_readStartup

set stageWidth [expr $ttbx-32]
set stageHeight [expr $ttby-200]

set itemWidth 128
set itemHeight 80

set itemXMax [expr $stageWidth/$itemWidth]

image create photo jfLogo -file "$ttblibPath/icons/system/JFLOGO.gif"
image create photo ttbLogo -file "$ttblibPath/icons/system/$ttblogo.gif"
image create photo NeXTkill -file "$ttblibPath/icons/system/Kill.gif"
image create photo NeXTarrowR -file "$ttblibPath/icons/system/Arrowright.gif"

wm title . "File viewer"
wm minsize . 640 480
wm geometry . [format %dx%d $ttbx $ttby]
wm group . .
wm iconname . "FileViewer 1.3"
wm protocol . WM_DELETE_WINDOW {ttb_exit}

# . configure -class FileViewer

listbox .hidedBuffer -selectborderwidth 0
listbox .hidedBuffer2 -selectborderwidth 0

frame .spacerTop -height 8 -width 100 -relief flat -bd 0 -bg $panelColor
frame .spacerLeft -width 8 -height 300 -relief flat -bd 0 -bg $panelColor

frame .container -relief flat -bd 0 -bg $panelColor -width $stageWidth -height [expr $ttby-8]


	frame .container.shortcuts -bg $panelColor -relief flat -bd 0 -width 640 -height 96
	bind .container.shortcuts <ButtonPress-3> "\
		ttb_showPopup top %X %Y {}"
	bind .container.shortcuts <Shift-ButtonPress-3> "\
		ttb_showPopup top %X %Y {}"

	canvas .container.folderStage \
		-xscrollcommand ".container.folderScroll set" \
		-bd 2 -relief sunken -height 76 -width 640 -bg $panelColor

	scrollbar .container.folderScroll -command ".container.folderStage xview" -orient horizontal

	frame .spacerMiddle -width 50 -height 8 -relief flat -bd 0 -bg $panelColor

	scrollbar .container.vscroll -command ".container.stage yview" -orient vertical
	scrollbar .container.hscroll -command ".container.stage xview" -orient horizontal
	canvas .container.stage \
		-yscrollcommand ".container.vscroll set" -xscrollcommand ".container.hscroll set" \
		-bd 2 -relief sunken -bg $stageColor

frame .spacerRight -width 8 -height 300 -relief flat -bd 0 -bg $panelColor

source $ttblibPath/functions/ttb_makeItems.tcl
source $ttblibPath/functions/ttb_item.tcl
source $ttblibPath/functions/ttb_if2.tcl

ttb_makeDefaultItem
ttb_readItems 00startup master

ttb_showFiles 0 ~/

# wm maxsize . [expr $ttbx+1] [expr $ttby+1]

# ttb_debug_selection

pack .spacerTop   -side top -fill x
pack .spacerLeft -side left -fill both
pack .spacerRight -side right -fill both
pack .container -side right -fill both -expand 1

pack propagate .container false
pack .container.shortcuts -in .container -fill both
pack .container.folderStage -in .container -fill both
pack .container.folderScroll -in .container -fill x
pack .spacerMiddle -in .container -fill y
pack .container.vscroll -in .container -side left -fill y
pack .container.hscroll -in .container -side bottom -fill x
pack .container.stage -expand 1 -in .container -fill both

ttb_commonBinding
source $ttblibPath/functions/ttb_dialog.tcl
source $ttblibPath/functions/ttb_procList.tcl
source $ttblibPath/functions/ttb_clipBoard.tcl
source $ttblibPath/functions/ttb_TTB.tcl
source $ttblibPath/functions/ttb_inspector.tcl

