#!/bin/sh
# the next line restarts using wish \
exec wishstep8.0 "$0" "$@"

proc ttb_about {} {

	ttb_dialog .about "about Recycler" \
		"Jun's STEP trash utility\n\nver 1.0\n\nJun Wakaya 1998."\
		jfLogo -32768 0 0 Ok
}

proc trash_icn {} {

	global recMode

	switch $recMode {
		Recycle {
			set lst [llength [glob ~/.trash/*]]

			if {$lst == 1} {
				.base configure -image RecyclerImage
			} else {
				.base configure -image RecyclerImage2
			}
		}

		Dispose {
			.base configure -image RecyclerImage3
		}
	}
}

proc trash_open {} {

	toplevel .cpan -border 10

	wm title .cpan "Inside trash :-)"

	frame .cpan.fr
	pack .cpan.fr
	listbox .cpan.fr.lb -width 32 -height 8 -xscrollcommand ".cpan.sbx set" -yscrollcommand ".cpan.fr.sby set"

	.cpan.fr.lb configure -selectmode extended

	scrollbar .cpan.fr.sby -command ".cpan.fr.lb yview"
	scrollbar .cpan.sbx -orient horizontal -command ".cpan.fr.lb xview"

	pack .cpan.fr.sby .cpan.fr.lb -in .cpan.fr -fill both -side left
	pack .cpan.sbx -fill both -side top

	frame .cpan.fr2 -relief flat -bd 5
	pack .cpan.fr2 -fill both
	button .cpan.fr2.cancel -text "cancel" -command "destroy .cpan"
	button .cpan.fr2.resume -text "resume" -command "trash_manupirate r"
	button .cpan.fr2.delete -text "delete" -command "trash_manupirate d"

	pack .cpan.fr2.cancel .cpan.fr2.resume .cpan.fr2.delete -side left -fill both

	frame .cpan.fr3 -relief flat -bd 5
	pack .cpan.fr3 -fill both

	button .cpan.fr3.update -text "update" -command "trash_reshow"
	button .cpan.fr3.about -text "about" -command "ttb_about"
	button .cpan.fr3.exit -text "exit" -command "exit"

	pack .cpan.fr3.update .cpan.fr3.about .cpan.fr3.exit -side left -fill both

	trash_reshow
}

proc trash_reshow {} {

	global recMode

	.cpan.fr.lb delete 0 end

	set fp [open ~/.trash/index.db r]

	set i 0

	while {![eof $fp]} {
	        set line$i [gets $fp]
		incr i
	}
	close $fp

	incr i -1

	for {set j 0} {$j < $i} {incr j} {
		eval set temp \$line$j
		set f [string first / $temp]
		.cpan.fr.lb insert end [string range $temp $f end]
	}

	.cpan.fr.lb xview moveto 1.0

	set lst [llength [glob ~/.trash/*]]

	if {$recMode=="Recycle"} {
		if {$lst == 1} {
			.base configure -image RecyclerImage
		} else {
			.base configure -image RecyclerImage2
		}
	}
}

proc trash_manupirate {mode} {

	set selection [.cpan.fr.lb curselection]
	if {$selection==""} return

	set fp [open ~/.trash/index.db r]

	set i 0

	while {![eof $fp]} {
	        set line$i [gets $fp]
		incr i
	}
	close $fp

	incr i -1

	set resumeList {}

	for {set j 0} {$j < $i} {incr j} {
		if {[string first $j $selection]!=-1} {
			eval set resumeList \"$resumeList \$line$j\"
		}
	}

	switch $mode {
		d {
			foreach k $resumeList {
				set f [string first / $k]
				set fname1 [string range $k 0 [expr $f-1]]
				file delete -force -- ~/.trash/$fname1
			}
		}


		r {
			foreach k $resumeList {
				set f [string first / $k]
				set fname1 [string range $k 0 [expr $f-1]]
				set fname2 [string range $k $f end]
				file rename -force -- ~/.trash/$fname1 $fname2
			}
		}
	}

	set fp [open "~/.trash/index.db" w]

	for {set l 0} {$l<$i} {incr l} {
		eval set temp \$line$l

		if {[string first $l $selection]==-1} {
			puts $fp $temp
		}
	}

	close $fp

	.cpan.fr.lb delete 0 end

	trash_reshow
}

proc trash_doDelete fname {

	global recMode

	set fname [join [split $fname \n]]

	if {$recMode == "Recycle"} {
		.base configure -image RecyclerImage2

		foreach z $fname {

			if {[llength [glob ~/.trash/*]] > 1} {
				eval set number [llength [glob ~/.trash/\[0123456789\]*]]
			} else {
				set number 0
			}

			file rename -force -- $z ~/.trash/[incr number]

			set db [open "~/.trash/index.db" r]
			set i 0

			while {![eof $db]} {
			        set line$i [gets $db]
				incr i
			}
			close $db

			incr i -1

			set db [open "~/.trash/index.db" w]

			for {set j 0} {$j < $i} {incr j} {
				eval puts \$db \$line$j
			}
			incr j
			puts $db "$j$z"

			close $db
		}
	} else {
		foreach z $fname {
			file delete -force -- $z
		}
	}
}

proc trash_truncateList {list} {

	set retList {}

	foreach i $list {
		if {$i == {}} {
			continue
		}

		catch [set retList [concat $retList $i]]
	}

	return $retList
}

proc trash_resize {toplevel} {

	switch $toplevel {
		root {
			. configure -width 64 -height 64
			wm maxsize . 64 64
			wm minsize . 64 64
			wm overrideredirect . true
			wm geometry . -2-0
		}
	}
}

proc trash_rootCreate {ttblibPath} {
	image create photo RecyclerImage -file $ttblibPath/icons/system/Recycler.gif
	image create photo RecyclerImage2 -file $ttblibPath/icons/system/RecycleLogo.gif
	image create photo RecyclerImage3 -file $ttblibPath/icons/system/BlackHoleGreenEdge.gif 

	image create photo jfLogo -file "$ttblibPath/icons/system/JFLOGO.gif"

	label .base -bd 0 -highlightthickness 0

	if {![file exists "~/.trash"]} {
		file mkdir "~/.trash"
		cd "~/.trash"
		exec touch ./index.db
	}

	set lst [llength [glob ~/.trash/*]]

	if {$lst == 1} {
		.base configure -image RecyclerImage
	} else {
		.base configure -image RecyclerImage2
	}

	pack .base -fill both
	pack propagate .base false
}

proc trash_bind {} {
	bind . <ButtonPress-1> "raise ."

	bind . <Double-ButtonPress-1> "trash_open"
	bind . <ButtonPress-3> "ttb_showPopup top %X %Y {}"

	bind . <Drop> "trash_doDelete %D"
}

proc ttb_showPopup {mode x y fname} {
	global ttbsendList ttbbookmarkList filt recMode

	if {[winfo exists .ttb_popup]} {
		destroy .ttb_popup
	}

	menu .ttb_popup -tearoff 0
	menu .ttb_popup.sub

	.ttb_popup.sub add radiobutton -command "trash_icn" -variable recMode -label "Recycle"
	.ttb_popup.sub add radiobutton -command "trash_icn" -variable recMode -label "Dispose"

	switch $mode {
		"top" {
			.ttb_popup add command -label "about Recycler" -command ttb_about
			.ttb_popup add cascade -label "Modes" -menu .ttb_popup.sub
			.ttb_popup add command -command "trash_open" -label "Control panel"
			.ttb_popup add command -label "exit" -command exit
		}

	}
	tk_popup .ttb_popup [expr $x-64] [expr $y-64]
}

#################################
# reading startup file
#
proc ttb_readStartup {} {
	global ttblibPath ttblogo stageColor panelColor

	if [file exists ~/GNUstep/Library/JunsSTEP/ttbinitrc] {
		set ttbrc [open ~/GNUstep/Library/JunsSTEP/ttbinitrc r]
		set line [split [gets $ttbrc]]

		while {![eof $ttbrc]} {
			switch [lindex $line 0] {
				ttblogo {
					set ttblogo [lindex $line 1]
				}
				ttbbg {
					set stageColor [lindex $line 1]
				}
				ttbfg {
					set panelColor [lindex $line 1]
				}
			}
	        set line [split [gets $ttbrc]]
		}
		close $ttbrc
	} else {
		puts "JunsSTEP isn't installed."
		exit
	}
}

set ttblibPath $env(HOME)/GNUstep/Library/JunsSTEP
set ttblogo {}
set stageColor {}
set panelColor {}
set recMode Recycle

source $ttblibPath/functions/ttb_dialog.tcl

ttb_readStartup

trash_rootCreate $ttblibPath
trash_resize root
trash_bind
